﻿using Newtonsoft.Json;
using UnityEngine;

public class AngleTool : Gadget
{
    /// \copydoc Gadget.s_type
    [JsonProperty]
    protected static new string s_type = "AngleTool";

    //Variables for AngleMode distinction
    private int PointNr = 0;
    private readonly PointFact[] AnglePoints = new PointFact[3];

    //Vertices for the Curve
    private int curveDrawingVertexCount = 36;
    private float curveRadius;

    public override void _Hit(RaycastHit hit)
    {
        if (hit.transform.gameObject.layer != LayerMask.NameToLayer("Point"))
            return;

        PointFact tempFact = (PointFact) StageStatic.stage.factState[hit.transform.GetComponent<FactObject>().URI];
        switch (PointNr)
        {
            case 0: // first point selected
                AnglePoints[PointNr++] = tempFact;
                break;

            case 1: //Check if the 2 selected points are the same
                if (AnglePoints[0].Id != tempFact.Id)
                {
                    AnglePoints[PointNr++] = tempFact;
                    ActivateLineDrawing();
                }
                break;

            case 2: //Create AngleFact
                //Check if new Point is equal to one of the previous points
                if (!(AnglePoints[0].Id == tempFact.Id || AnglePoints[1].Id == tempFact.Id))
                {
                    FactManager.AddAngleFact(AnglePoints[0].Id, AnglePoints[1].Id, tempFact.Id);
                    ResetGadget();
                    return;
                }
                break;
        }
    }

    protected override void _ResetGadget()
    {
        PointNr = 0;
    }

    //Expect a LineFact here, where Line.Pid2 will be the Basis-Point of the angle
    protected override void _ActivateLineDrawing()
    {
        //In AngleMode with 3 Points we want to draw nearly a rectangle so we add a startPoint and an Endpoint to this preview
        GadgetBehaviour.LineRenderer.positionCount = curveDrawingVertexCount + 2;

        GadgetBehaviour.LineRenderer.startWidth = 0.05f;
        GadgetBehaviour.LineRenderer.endWidth = 0.05f;

        curveRadius = 0.3f * (AnglePoints[0].Point - AnglePoints[1].Point).magnitude;
    }

    protected override void _UpdateLineDrawing()
    {
        //Start: AngleMiddlePoint -> FirstPoint of Curve
        SetPosition(0, AnglePoints[1].Point);
        //End: LastPoint of Curve -> AngleMiddlePoint
        SetPosition(GadgetBehaviour.LineRenderer.positionCount - 1, AnglePoints[1].Point);

        //Circle: p + r(cost)v1 + r(sint)v2: t real, v1&v2 unit orthogonal, r radius
        Vector3 v1 = (GadgetBehaviour.Cursor.transform.position - AnglePoints[1].Point).normalized;
        Vector3 _v2 = (AnglePoints[0].Point - AnglePoints[1].Point);
        Vector3 v2 = (_v2 - Math3d.ProjectPointOnLine(Vector3.zero, v1, _v2)).normalized;
        float increment = 2 * Mathf.PI / (curveDrawingVertexCount-1) * Vector3.Angle(v1, _v2) / 360;
        float percentage = 0;
        for (int i = 1; i < curveDrawingVertexCount + 1; i++, percentage += increment)
        {
            Vector3 frag = AnglePoints[1].Point
                + curveRadius * (Mathf.Cos(percentage) * v1 + Mathf.Sin(percentage) * v2);
            SetPosition(i, frag);
        }
    }
}
